#include "common.h"
#include "Material.h"
#include "Texture.h"
#include "glInterface.h"


namespace CHCDemo
{

RgbColor RandomColor(float a, float b)
{
	return RgbColor(a + Random(b), a + Random(b), a + Random(b));
}


void Material::InitMaterial()
{
	mTexture = NULL;

	mAmbientColor = RgbColor(.2f, .2f, .2f);
	mDiffuseColor = RgbColor(1.0f, 1.0f, 1.0f);
	mSpecularColor = RgbColor(0.0f, 0.0f, 0.0f);
	mSpecularColor = RgbColor(1.0f, 1.0f, 1.0f);
}


Material::Material(): mId(0)
{
	InitMaterial();
}


Material::Material(int id): 
mId(id)
{
	InitMaterial();
}


Material::Material(const RgbColor &color):
mDiffuseColor(color),
mAmbientColor(color),
mSpecularColor(0, 0, 0), 
mId(0),
mTexture(NULL)
{
}


int Material::GetId() const
{
	return mId;
}


Material RandomMaterial()
{
	float a = 0.1f;
	float b = 0.9f;

	Material m;
	m.mDiffuseColor = RandomColor(a, b);

	return m;
}


void Material::Render()
{
	if (mTexture)
		mTexture->Bind();
	else
		glBindTexture(GL_TEXTURE_2D, 0);

/*	float amb[] = {0.2f, 0.2f, 0.2f, 1.0f};
	float dif[] = {1.0f, 1.0f, 1.0f, 1.0f};
	float spec[] = {1.0f, 1.0f, 1.0f, 1.0f};

	glMaterialfv(GL_FRONT, GL_AMBIENT, (float *)&amb);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, (float *)&dif);
	glMaterialfv(GL_FRONT, GL_SPECULAR, (float *)&spec);
*/
	glMaterialfv(GL_FRONT, GL_AMBIENT, (float *)&mAmbientColor.r);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, (float *)&mDiffuseColor.r);
	glMaterialfv(GL_FRONT, GL_SPECULAR, (float *)&mSpecularColor.r);
}


}
