#include "RenderQueue.h"
#include "SceneEntity.h"
#include "Geometry.h"
#include "Texture.h"
#include "Material.h"



namespace CHCDemo
{


inline static bool IsLower(SceneEntity *ent1, SceneEntity *ent2)
{
	if (ent1->GetGeometry()->HasTexture() && ent2->GetGeometry()->HasTexture())
	{
		Texture *t1 = ent1->GetMaterial()->GetTexture();
		Texture *t2 = ent1->GetMaterial()->GetTexture();

		return t1->GetByteSize() < t2->GetByteSize();
	}
	else if (ent2->GetGeometry()->HasTexture())
	{
		return true;
	}

	return false;
}



RenderQueue::RenderQueue(): mState(NULL)
{
}


RenderQueue::RenderQueue(RenderState *state):
mState(state)
{
}


void RenderQueue::Enqueue(SceneEntity **entities, int size)
{
	for (int i = 0; i < size; ++ i)
		mEntities.push_back(entities[i]);

}
	

void RenderQueue::Clear()
{
	mEntities.clear();

}


void RenderQueue::SetRenderState(RenderState *state)
{
	mState = state;
}


void RenderQueue::Render()
{
	Sort();

	SceneEntityContainer::const_iterator sit, sit_end = mEntities.end();

	for (sit = mEntities.begin(); sit != sit_end; ++ sit)
	{
		SceneEntity *ent = *sit;
		ent->Render(mState);
	}
}

	

void RenderQueue::Sort()
{
	sort(mEntities.begin(), mEntities.end());
}


}