#include "Visualization.h"
#include "glInterface.h"
#include "Camera.h"
#include "SceneEntity.h"
#include "Bvh.h"
#include <stack>


using namespace std;


namespace CHCDemo
{

/******************************************************/
/*           Vizualization implementation           */
/******************************************************/


Visualization::Visualization(Bvh *bvh, 
							 Camera *camera, 
							 Camera *vizCamera, 
							 RenderState *renderState): 
mBvh(bvh), mCamera(camera), mVizCamera(vizCamera), mRenderState(renderState)
{
}


void Visualization::SetHierarchy(Bvh *bvh)
{
	mBvh = bvh;
}


void Visualization::SetRenderState(RenderState *state)
{
	mRenderState = state;
}


void Visualization::Render()
{
	stack<BvhNode *> tStack;
	tStack.push(mBvh->GetRoot());

	while (!tStack.empty())
	{
		BvhNode *node = tStack.top();
		tStack.pop();

		if (!node->IsLeaf())
		{
			BvhInterior *interior = static_cast<BvhInterior *>(node);

			tStack.push(interior->GetFront());
			tStack.push(interior->GetBack());
		}
		else
		{
			//leaves.push_back(static_cast<BvhLeaf *>(node));
		}
	}
}


void Visualization::RenderBox(const AxisAlignedBox3 &box)
{
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glDisable(GL_CULL_FACE);
	glColor3f(1.0f, 0.0f, 0.0f);

	glBegin(GL_QUADS);
	glVertex3f(box.Min().x, box.Max().y, box.Min().z);
	glVertex3f(box.Max().x, box.Max().y, box.Min().z);
	glVertex3f(box.Max().x, box.Min().y, box.Min().z);
	glVertex3f(box.Min().x, box.Min().y, box.Min().z);
	glEnd();

	glBegin(GL_QUADS);
	glVertex3f(box.Min().x, box.Min().y, box.Max().z);
	glVertex3f(box.Max().x, box.Min().y, box.Max().z);
	glVertex3f(box.Max().x, box.Max().y, box.Max().z);
	glVertex3f(box.Min().x, box.Max().y, box.Max().z);
	glEnd();

	glBegin(GL_QUADS);
	glVertex3f(box.Max().x, box.Min().y, box.Min().z);
	glVertex3f(box.Max().x, box.Min().y, box.Max().z);
	glVertex3f(box.Max().x, box.Max().y, box.Max().z);
	glVertex3f(box.Max().x, box.Max().y, box.Min().z);
	glEnd();

	glBegin(GL_QUADS);
	glVertex3f(box.Min().x, box.Min().y, box.Min().z);
	glVertex3f(box.Min().x, box.Min().y, box.Max().z);
	glVertex3f(box.Min().x, box.Max().y, box.Max().z);
	glVertex3f(box.Min().x, box.Max().y, box.Min().z);
	glEnd();

	glBegin(GL_QUADS);
	glVertex3f(box.Min().x, box.Min().y, box.Min().z);
	glVertex3f(box.Max().x, box.Min().y, box.Min().z);
	glVertex3f(box.Max().x, box.Min().y, box.Max().z);
	glVertex3f(box.Min().x, box.Min().y, box.Max().z);
	glEnd();

	glBegin(GL_QUADS);
	glVertex3f(box.Min().x, box.Max().y, box.Min().z);
	glVertex3f(box.Max().x, box.Max().y, box.Min().z);
	glVertex3f(box.Max().x, box.Max().y, box.Max().z);
	glVertex3f(box.Min().x, box.Max().y, box.Max().z);
	glEnd();

	glEnable(GL_CULL_FACE);

	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
}


void Visualization::RenderFrustum()
{
	Vector3 ftl, ftr, fbl, fbr, ntl, ntr, nbl, nbr;
	mCamera->ComputePoints(ftl, ftr, fbl, fbr, ntl, ntr, nbl, nbr);

	glLineWidth(2);

	glBegin(GL_LINE_LOOP);
	glVertex3d(fbl.x, fbl.y, fbl.z);
	glVertex3d(fbr.x, fbr.y, fbr.z);
	glVertex3d(ftr.x, ftr.y, ftr.z);
	glVertex3d(ftl.x, ftl.y, ftl.z);
	glEnd();

	glBegin(GL_LINE_LOOP);
	glVertex3d(nbl.x, nbl.y, nbl.z);
	glVertex3d(nbr.x, nbr.y, nbr.z);
	glVertex3d(ntr.x, ntr.y, ntr.z);
	glVertex3d(ntl.x, ntl.y, ntl.z);
	glEnd();

	glBegin(GL_LINE_LOOP);
	glVertex3d(fbl.x, fbl.y, fbl.z);
	glVertex3d(ftl.x, ftl.y, ftl.z);
	glVertex3d(ntl.x, ntl.y, ntl.z);
	glVertex3d(nbl.x, nbl.y, nbl.z);
	glEnd();

	glBegin(GL_LINE_LOOP);
	glVertex3d(fbr.x, fbr.y, fbr.z);
	glVertex3d(ftr.x, ftr.y, ftr.z);
	glVertex3d(ntr.x, ntr.y, ntr.z);
	glVertex3d(nbr.x, nbr.y, nbr.z);
	glEnd();

	glBegin(GL_LINE_LOOP);
	glVertex3d(fbr.x, fbr.y, fbr.z);
	glVertex3d(fbl.x, fbl.y, fbl.z);
	glVertex3d(nbl.x, nbl.y, nbl.z);
	glVertex3d(nbr.x, nbr.y, nbr.z);
	glEnd();

	glBegin(GL_LINE_LOOP);
	glVertex3d(ftr.x, ftr.y, ftr.z);
	glVertex3d(ftl.x, ftl.y, ftl.z);
	glVertex3d(ntl.x, ntl.y, ntl.z);
	glVertex3d(ntr.x, ntr.y, ntr.z);
	glEnd();
}


}